@@ -5,6 +5,8 @@ from bokeh.server.server import Server
 
 
 class BokehServer(object):
+    server_kwargs = {}
+
     def listen(self, addr):
         if self.server:
             return
@@ -20,11 +22,13 @@ class BokehServer(object):
                 else:
                     kwargs = {}
 
+                kwargs.update(self.server_kwargs)
+
                 self.server = Server(self.apps, io_loop=self.loop,
                                      port=port, address=ip,
                                      check_unused_sessions_milliseconds=500,
-                                     allow_websocket_origin=["*"], **kwargs
-                                     )
+                                     allow_websocket_origin=["*"],
+                                     **kwargs)
                 if bokeh.__version__ <= '0.12.3':
                     self.server.start(start_loop=False)
                 else:
@@ -49,45 +53,6 @@ class BokehServer(object):
         if self.server._tornado._ping_job is not None:
             self.server._tornado._ping_job.stop()
 
-        # self.server.stop()
         # https://github.com/bokeh/bokeh/issues/5494
-
-
-def format_bytes(n):
-    """ Format bytes as text
-
-    >>> format_bytes(1)
-    '1 B'
-    >>> format_bytes(1234)
-    '1.23 kB'
-    >>> format_bytes(12345678)
-    '12.35 MB'
-    >>> format_bytes(1234567890)
-    '1.23 GB'
-    """
-    if n > 1e9:
-        return '%0.2f GB' % (n / 1e9)
-    if n > 1e6:
-        return '%0.2f MB' % (n / 1e6)
-    if n > 1e3:
-        return '%0.2f kB' % (n / 1000)
-    return '%d B' % n
-
-
-def format_time(n):
-    """ format integers as time
-
-    >>> format_time(1)
-    '1.00 s'
-    >>> format_time(0.001234)
-    '1.23 ms'
-    >>> format_time(0.00012345)
-    '123.45 us'
-    >>> format_time(123.456)
-    '123.46 s'
-    """
-    if n >= 1:
-        return '%.2f s' % n
-    if n >= 1e-3:
-        return '%.2f ms' % (n * 1e3)
-    return '%.2f us' % (n * 1e6)
+        if bokeh.__version__ >= '0.12.4':
+            self.server.stop()
