@@ -26,12 +26,12 @@ except ImportError:
     np = False
 
 from . import components
-from .components import DashboardComponent
+from .components import DashboardComponent, ProfileTimePlot
 from .core import BokehServer
-from .worker import SystemMonitor, format_time, counters_doc
+from .worker import SystemMonitor, counters_doc
 from .utils import transpose
 from ..metrics import time
-from ..utils import log_errors, format_bytes
+from ..utils import log_errors, format_bytes, format_time
 from ..diagnostics.progress_stream import color_of, progress_quads, nbytes_bar
 from ..diagnostics.progress import AllProgress
 from .task_stream import TaskStreamPlugin
@@ -53,7 +53,8 @@ with open(os.path.join(os.path.dirname(__file__), 'template.html')) as f:
 
 template = jinja2.Template(template_source)
 
-template_variables = {'pages': ['status', 'workers', 'tasks', 'system', 'counters']}
+template_variables = {'pages': ['status', 'workers', 'tasks', 'system',
+                                'profile', 'counters']}
 
 
 def update(source, data):
@@ -135,8 +136,9 @@ class Occupancy(DashboardComponent):
 
             fig = figure(title='Occupancy', tools='', id='bk-occupancy-plot',
                          x_axis_type='datetime', **kwargs)
-            fig.rect(source=self.source, x='x', width='ms', y='y', height=1,
-                     color='color')
+            rect = fig.rect(source=self.source, x='x', width='ms', y='y', height=1,
+                            color='color')
+            rect.nonselection_glyph = None
 
             fig.xaxis.minor_tick_line_alpha = 0
             fig.yaxis.visible = False
@@ -283,19 +285,22 @@ class CurrentLoad(DashboardComponent):
 
             processing = figure(title='Tasks Processing', tools='', id='bk-nprocessing-plot',
                                 width=int(width / 2), **kwargs)
-            processing.rect(source=self.source,
-                            x='nprocessing-half', y='y',
-                            width='nprocessing', height=1,
-                            color='nprocessing-color')
+            rect = processing.rect(source=self.source,
+                                   x='nprocessing-half', y='y',
+                                   width='nprocessing', height=1,
+                                   color='nprocessing-color')
             processing.x_range.start = 0
+            rect.nonselection_glyph = None
 
             nbytes = figure(title='Bytes stored', tools='',
                             id='bk-nbytes-worker-plot', width=int(width / 2),
                             **kwargs)
-            nbytes.rect(source=self.source,
-                        x='nbytes-half', y='y',
-                        width='nbytes', height=1,
-                        color='nbytes-color')
+            rect = nbytes.rect(source=self.source,
+                               x='nbytes-half', y='y',
+                               width='nbytes', height=1,
+                               color='nbytes-color')
+            rect.nonselection_glyph = None
+
             nbytes.axis[0].ticker = BasicTicker(mantissas=[1, 256, 512], base=1024)
             nbytes.xaxis[0].formatter = NumeralTickFormatter(format='0.0 b')
             nbytes.xaxis.major_label_orientation = -math.pi / 12
@@ -994,6 +999,18 @@ def status_doc(scheduler, extra, doc):
         doc.template_variables.update(extra)
 
 
+def profile_doc(scheduler, extra, doc):
+    with log_errors():
+        doc.title = "Dask Profile"
+        prof = ProfileTimePlot(scheduler, sizing_mode='scale_width', doc=doc)
+        doc.add_root(prof.root)
+        doc.template = template
+        doc.template_variables['active_page'] = 'profile'
+        doc.template_variables.update(extra)
+
+        prof.trigger_update()
+
+
 class BokehScheduler(BokehServer):
     def __init__(self, scheduler, io_loop=None, prefix='', **kwargs):
         self.scheduler = scheduler
@@ -1014,6 +1031,7 @@ class BokehScheduler(BokehServer):
         events = Application(FunctionHandler(partial(events_doc, scheduler, extra)))
         tasks = Application(FunctionHandler(partial(tasks_doc, scheduler, extra)))
         status = Application(FunctionHandler(partial(status_doc, scheduler, extra)))
+        profile = Application(FunctionHandler(partial(profile_doc, scheduler, extra)))
 
         self.apps = {
             '/system': systemmonitor,
@@ -1022,7 +1040,8 @@ class BokehScheduler(BokehServer):
             '/events': events,
             '/counters': counters,
             '/tasks': tasks,
-            '/status': status
+            '/status': status,
+            '/profile': profile,
         }
 
         self.loop = io_loop or scheduler.loop
