@@ -1,6 +1,6 @@
 import pytest
 
-from distributed.utils_test import gen_cluster, inc, div, loop
+from distributed.utils_test import div, gen_cluster, inc
 from distributed import as_completed, Client
 
 
@@ -8,7 +8,7 @@ from distributed import as_completed, Client
 def test_await_future(c, s, a, b):
     future = c.submit(inc, 1)
 
-    async def f():
+    async def f():  # flake8: noqa
         result = await future
         assert result == 2
 
@@ -39,10 +39,27 @@ def test_as_completed_async_for(c, s, a, b):
     assert set(results) == set(range(1, 11))
 
 
+@gen_cluster(client=True)
+def test_as_completed_async_for_results(c, s, a, b):
+    futures = c.map(inc, range(10))
+    ac = as_completed(futures, with_results=True)
+    results = []
+
+    async def f():
+        async for future, result in ac:
+            results.append(result)
+
+    yield f()
+
+    assert set(results) == set(range(1, 11))
+    assert not s.counters['op'].components[0]['gather']
+
+
 def test_async_with(loop):
     result = None
     client = None
     cluster = None
+
     async def f():
         async with Client(processes=False, asynchronous=True) as c:
             nonlocal result, client, cluster
