@@ -8,20 +8,20 @@ from operator import add, mul
 import sys
 
 from dask import delayed
-from toolz import merge, concat, valmap, first, frequencies
+from toolz import merge, concat, valmap, first, frequencies, pluck
 from tornado import gen
 
 import pytest
 
-from distributed import Nanny, Worker, Client, wait
+from distributed import Nanny, Worker, Client, wait, fire_and_forget
 from distributed.core import connect, rpc, CommClosedError
 from distributed.scheduler import validate_state, Scheduler, BANDWIDTH
 from distributed.client import wait
 from distributed.metrics import time
 from distributed.protocol.pickle import dumps
 from distributed.worker import dumps_function, dumps_task
-from distributed.utils_test import (inc, dec, gen_cluster, gen_test,
-                                    readone, slowinc, slowadd, cluster, div)
+from distributed.utils_test import (inc, dec, gen_cluster, gen_test, readone,
+                                    slowinc, slowadd, slowdec, cluster, div)
 from distributed.utils_test import loop # flake8: noqa
 from distributed.utils import tmpfile
 from distributed.utils_test import slow
@@ -1122,6 +1122,8 @@ def test_correct_bad_time_estimate(c, s, *workers):
     assert all(w.data for w in workers)
 
 
+@pytest.mark.skipif(not sys.platform.startswith('linux'),
+                    reason="Need 127.0.0.2 to mean localhost")
 @gen_test(timeout=None)
 def test_service_hosts_match_scheduler():
     from distributed.http.scheduler import HTTPScheduler
@@ -1141,3 +1143,45 @@ def test_service_hosts_match_scheduler():
         sock = first(s.services['http']._sockets.values())
         assert sock.getsockname()[0] == '127.0.0.2'
         yield s.close()
+
+
+@gen_cluster(client=True, worker_kwargs={'profile_cycle_interval': 100})
+def test_profile_metadata(c, s, a, b):
+    start = time() - 1
+    futures = c.map(slowinc, range(10), delay=0.05, workers=a.address)
+    yield wait(futures)
+    yield gen.sleep(0.200)
+
+    meta = yield s.get_profile_metadata(profile_cycle_interval=0.100)
+    now = time()
+    assert meta
+    assert all(start < t < now for t, count in meta['counts'])
+    assert all(0 < count < 11 for t, count in meta['counts'][:4])
+    assert not meta['counts'][-1][1]
+
+
+@gen_cluster(client=True, worker_kwargs={'profile_cycle_interval': 100})
+def test_profile_metadata_keys(c, s, a, b):
+    start = time() - 1
+    x = c.map(slowinc, range(10), delay=0.05)
+    y = c.map(slowdec, range(10), delay=0.05)
+    yield wait(x + y)
+
+    meta = yield s.get_profile_metadata(profile_cycle_interval=0.100)
+    assert set(meta['keys']) == {'slowinc', 'slowdec'}
+    assert len(meta['counts']) == len(meta['keys']['slowinc'])
+
+
+@gen_cluster(client=True)
+def test_cancel_fire_and_forget(c, s, a, b):
+    x = delayed(slowinc)(1, delay=0.05)
+    y = delayed(slowinc)(x, delay=0.05)
+    z = delayed(slowinc)(y, delay=0.05)
+    w = delayed(slowinc)(z, delay=0.05)
+    future = c.compute(w)
+    fire_and_forget(future)
+
+    yield gen.sleep(0.05)
+    yield future.cancel(force=True)
+    assert future.status == 'cancelled'
+    assert not s.task_state
